/* ybasicw.h - definitions for the wxWidgets ybasic gui
 *
 * Copyright (c) 2009 by David Michael Betz.  All rights reserved.
 *
 */

#ifndef __YBASICW_H__
#define __YBASICW_H__

#include "wx/wx.h"
#include "wx/config.h"

#include "main.h"
#include "listener.h"

extern "C" {    
    #include "../runtime/db_vm.h"
}

#define NUM_PORTS   69

enum {
    ID_PORT_COM1 = wxID_HIGHEST + 1,
    ID_LOAD = ID_PORT_COM1 + NUM_PORTS,
    ID_CHAMELEON_AVR,
    ID_CHAMELEON_PIC,
    ID_XGS_AVR,
    ID_XGS_PIC,
    ID_RELOAD
};

class Basic : public wxApp
{
public:

    void ConsolePutC(int ch);
    void ConsoleFlushOutput();
    void ConsoleGetLine();

    void Warning(char *fmt, ...);

    void SaveSettingsAndExit();

    void OpenPort(const char *port, bool init);
    void ClosePort();

    void SetBoardType(int boardType);

    void CompileAndLoad();
    void ReloadLastFile();

    virtual bool OnInit();

private:

    void SizeAndPositionWindows();
    void SaveWindowPositions();
    void SaveWindowPosition(wxString key,wxTopLevelWindow *win);
    void RestoreSettings();
    void RestoreWindowPositions();
    void RestoreWindowPosition(wxString key,wxTopLevelWindow *win);

    static void HandleInput(void *data, const char *buf);

    void Load(wxString file);
    void Download(wxString file);
    
    wxConfig *m_config;
    wxRect m_screenRect;
    MainFrame *m_main;

    bool m_portOpen;
    wxString m_port;

    int m_boardType;

    wxString m_lastFileLoaded;
};

DECLARE_APP(Basic)

#endif
