include "chameleon.bas"
include "ntsc_drv.bas"
include "gfx_drv.bas"
include "key_drv.bas"

// eliminate the string heap to save space
option heap = "0:0"

def worldw = 20
def worldh = 20

def topmargin = 1
def leftmargin = 8

def blankTile = 144
def liveTile = blankTile + 1

def myTileMap1 = 0
def myTileMap2 = 1
def myPalette = 2
def myBlankTile = myPalette * 256 + blankTile
def myLiveTile = myPalette * 256 + liveTile

dim blank_tile_bits(8) = {
  0x0000
  0x0000
  0x0000
  0x0000
  0x0000
  0x0000
  0x0000
  0x0000
}

dim live_tile_bits(8) = {
  0x03f0
  0x0ffc
  0x3fff
  0x3fff
  0x3fff
  0x0ffc
  0x03f0
  0x0000
}

dim palette(4) as byte = {
  0x03
  0x04
  0x05
  0x07
}

def arrayw = 1 + worldw + 1 + worldw + 1
def arrayh = 1 + worldh + 1

def world1 = 1
def world2 = 1 + worldw + 1

dim world(arrayw, arrayh) as byte
dim thisx, nextx

dim drawTileMap, displayTileMap

def visit(x, y)
  visit = 0
  for y1 = y-1 to y+1
    for x1 = x-1 to x+1
      if world(x1, y1) > 0 then
        visit = visit + 1
      end if
    next x1
  next y1
end def

sub clearall
  for y1 = 1 to arrayh
    for x1 = 1 to arrayw
      world(x1, y1) = 0
    next x1
  next y1
end sub

sub fill
  for y1 = 2 to worldh+1
    for x1 = 1 to worldw
      if rnd(4) = 3 then
        cell = 1
      else
        cell = 0
      end if
      world(thisx+x1, y1) = cell
    next x1
  next y1
end sub

sub nextgen
  for y1 = 2 to worldh+1
    for x1 = 1 to worldw
      neighbors = visit(thisx+x1, y1)
      if world(thisx+x1, y1) = 0 then
        if neighbors = 3 then
          new = 1
        else
          new = 0
        end if
      else
        if neighbors = 3 or neighbors = 4 then
          new = 1
        else
          new = 0
        end if
      end if
      world(nextx+x1, y1) = new
    next x1
  next y1
end sub

sub showgen(map)
  for y1 = 1 to worldh
    for x1 = 1 to worldw
      if world(thisx+x1, 1+y1) = 0 then
          tile = myBlankTile
      else
          tile = myLiveTile
      end if
      gfx_write_tile(map, tile, leftmargin+x1, topmargin+y1)
    next x1
  next y1
end sub

sub InitTileMap(map)
  gfx_set_term_tile_map(map)
  ntsc_cls
  ntsc_setxy(1, 1)
  display "Life!";
  ntsc_setxy(1, 3)
  display "Gen# ";
end sub

key_start

print "initializing graphics"

gfx_set_top_overscan_color(color_green)
gfx_set_bottom_overscan_color(color_green)

gfx_define_tile_map(myTileMap1, 0, 32, 24)
gfx_define_tile_map(myTileMap2, 32*24*2, 32, 24)

gfx_load_tile(blankTile, blank_tile_bits)
gfx_load_tile(liveTile, live_tile_bits)

gfx_load_palette(myPalette, palette)

print "initializing tile maps"

drawTileMap = myTileMap1
displayTileMap = myTileMap2

InitTileMap(drawTileMap)
InitTileMap(displayTileMap)

print "clearing worlds"
clearall

do
  thisx = world1
  nextx = world2
  print "filling world"
  fill

  generations = 30
  generation = 0

  do

    // setup the tile maps
    gfx_display_tile_map(displayTileMap, 0, 0)
    gfx_set_term_tile_map(drawTileMap)

    // count the generation
    generation = generation + 1
    print "generation "; generation
    ntsc_setxy(6, 3)
    display generation; " ";

    // wait for vsync to display the current generation
    // waitforvsync
    showgen(drawTileMap)
    
    // compute the next generation
    nextgen
    
    // swap worlds
    tmp = thisx
    thisx = nextx
    nextx = tmp

    // swap tile maps
    tmp = drawTileMap
    drawTileMap = displayTileMap
    displayTileMap = tmp

    // check for commands
    if key_ready then
      key = key_get
    else
      key = 0
    end if
    
    // determine if it's time to stop
    if generation >= generations then
      done = 1
    else
      done = 0
    end if

  loop until done

loop
