#include <stdlib.h>
#include "db_vm.h"

Interpreter interp;

int main(int argc, char *argv[])
{
    const uint8_t *image;
    uint8_t *data;
    uint16_t dataSize;

    VM_sysinit(argc, argv, &image, &data, &dataSize);

    VM_printf(str_program_id, FILE_VERSION >> 8, FILE_VERSION & 0xff);

    if (InitInterpreter(&interp, image, data, dataSize))
        Execute(&interp);

#ifndef WIN32
    for(;;)
        ;
#endif

    return 0;
}

void VM_printf(const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    VM_vprintf(fmt, ap);
    va_end(ap);
}

void VM_vprintf(const char *fmt, va_list ap)
{
    char buf[80], *p = buf;
    VMVSNPRINTF(buf, sizeof(buf), fmt, ap);
    while (*p != '\0')
        VM_putchar(*p++);
}

void VM_getline(char *buf, int size)
{
    int ch, max = size - 2, i = 0;

    VM_flush();

    for (;;) {
        switch (ch = VM_getchar()) {
        case '\r':
            VM_putchar('\r');
            VM_putchar('\n');
            buf[i++] = '\n';
            buf[i] = '\0';
            --size;
            return;
        case '\010':
        case '\177':
            if (i > 0) {
                VM_putchar('\010');
                VM_putchar(' ');
                VM_putchar('\010');
                --i;
            }
            break;
        default:
            if (i < max) {
                buf[i++] = ch;
                VM_putchar(ch);
            }
            break;
        }
    }
}
